import pytest

from bilili.api.danmaku import get_danmaku
from bilili.common.base import touch_dir
from bilili.video import BililiContainer


def test_danmaku():
    container = BililiContainer(
        id=1,
        name="Unknown",
        path=touch_dir("tmp/ThreeBody/Videos/video1"),
        meta={
            "avid": '',
            "bvid": '1i741187Dp',
            "cid": '149439373',
        },
        format=format,
    )
    get_danmaku(container)



import os
import re


class Task():
    """任务对象"""

    def __init__(self, func, args=(), kw={}):
        """接受函数与参数以初始化对象"""

        self.func = func
        self.args = args
        self.kw = kw

    def __call__(self):
        """执行函数
        同步函数直接执行并返回结果，异步函数返回该函数
        """

        result = self.func(*self.args, **self.kw)
        return result


class Writer():
    """ 文件写入器，持续打开文件对象，直到使用完毕后才关闭 """

    def __init__(self, path, mode='wb', **kw):
        self.path = path
        self._f = open(path, mode, **kw)

    def __del__(self):
        self._f.close()

    def flush(self):
        self._f.flush()

    def write(self, content):
        self._f.write(content)


class Text(Writer):
    """ 文本写入器 """

    def __init__(self, path, **kw):
        kw['encoding'] = kw.get('encoding', 'utf-8')
        super().__init__(path, 'w', **kw)

    def write_string(self, string):
        self.write(string + '\n')


def touch_dir(path):
    """ 若文件夹不存在则新建，并返回标准路径 """
    if not os.path.exists(path):
        os.makedirs(path)
    return os.path.normpath(path)


def touch_file(path):
    """ 若文件不存在则新建，并返回标准路径 """
    if not os.path.exists(path):
        open(path, 'w').close()
    return os.path.normpath(path)


def repair_filename(filename):
    """ 修复不合法的文件名 """
    def to_full_width_chr(matchobj):
        char = matchobj.group(0)
        full_width_char = chr(ord(char) + ord('？') - ord('?'))
        return full_width_char
    # 路径非法字符，转全角
    regex_path = re.compile(r'[\\/:*?"<>|]')
    # 空格类字符，转空格
    regex_spaces = re.compile(r'\s+')
    # 不可打印字符，移除
    regex_non_printable = re.compile(
        r'[\001\002\003\004\005\006\007\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f'
        r'\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a]')

    filename = regex_path.sub(to_full_width_chr, filename)
    filename = regex_spaces.sub(' ', filename)
    filename = regex_non_printable.sub('', filename)
    filename = filename.strip()
    return filename


def get_size(path):
    """ 获取文件夹或文件的字节数 """
    if os.path.isfile(path):
        return os.path.getsize(path)
    elif os.path.isdir(path):
        size = 0
        for subpath in os.listdir(path):
            size += get_size(os.path.join(path, subpath))
        return size
    else:
        return 0


def size_format(size, ndigits=2):
    """ 输入数据字节数，与保留小数位数，返回数据量字符串 """
    flag = '-' if size < 0 else ''
    size = abs(size)
    units = ["Bytes", "kB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB", "BB"]
    idx = len(units) - 1
    unit = ""
    unit_size = 0
    while idx >= 0:
        unit_size = 2 ** (idx * 10)
        if size >= unit_size:
            unit = units[idx]
            break
        idx -= 1
    return "{}{:.{}f} {}".format(flag, size/unit_size, ndigits, unit)


def get_string_width(string):
    """ 计算包含中文的字符串宽度 """
    try:
        length = len(string.encode('gbk'))
    except:
        length = len(string)
    return length


def remove(path):
    if os.path.isdir(path):
        for filename in os.listdir(path):
            remove(os.path.join(path, filename))
    elif os.path.isfile(path):
        os.remove(path)
    else:
        pass



import os
import re
import time
import requests
import cv2

from bilili.common.base import get_size, size_format, touch_dir, touch_file, Task
from bilili.common.thread import ThreadPool


def parse_episodes(episodes_str, total):
    """ 将选集字符串转为列表 """

    # 解析字符串为列表
    print("全 {} 话".format(total))
    if episodes_str == "all":
        episode_list = list(range(1, total+1))
    elif re.match(r"\d+~\d+", episodes_str):
        start, end = episodes_str.split("~")
        start, end = int(start), int(end)
        assert end > start, "终点值应大于起点值"
        episode_list = list(range(start, end+1))
    elif re.match(r"\d+(,\d+)*", episodes_str):
        episode_list = episodes_str.split(",")
        episode_list = list(map(int, episode_list))
    else:
        episode_list = []

    # 筛选满足条件的剧集
    out_of_range = []
    episodes = []
    for episode in episode_list:
        if episode in range(1, total+1):
            if episode not in episodes:
                episodes.append(episode)
        else:
            out_of_range.append(episode)
    if out_of_range:
        print("warn: 剧集 {} 不存在".format(",".join(list(map(str, out_of_range)))))

    print("已选择第 {} 话".format(",".join(list(map(str, episodes)))))
    assert episodes, "没有选中任何剧集"
    return episodes


def convert_danmaku(video_path_list):
    """ 将视频文件夹下的 xml 弹幕转换为 ass 弹幕 """
    # 检测插件是否已经就绪
    plugin_url = "https://raw.githubusercontent.com/m13253/danmaku2ass/master/danmaku2ass.py"
    plugin_path = "plugins/danmaku2ass.py"
    touch_dir(os.path.dirname(plugin_path))
    touch_file(os.path.join(os.path.dirname(plugin_path), "__init__.py"))
    if not os.path.exists(plugin_path):
        print("下载插件中……")
        res = requests.get(plugin_url)
        with open(plugin_path, "w", encoding="utf8") as f:
            f.write(res.text)

    # 使用插件进行转换
    from plugins.danmaku2ass import Danmaku2ASS
    for video_path in video_path_list:
        name = os.path.splitext(video_path)[0]
        print("convert {} ".format(os.path.split(name)[-1]), end="\r")
        if not os.path.exists(name+".mp4") or \
                not os.path.exists(name+".xml"):
            continue
        cap = cv2.VideoCapture(name+".mp4")
        __, frame = cap.read()
        h, w, __ = frame.shape
        Danmaku2ASS(
            name+".xml", "autodetect", name+".ass",
            w, h, reserve_blank=0,
            font_face=_('(FONT) sans-serif')[7:],
            font_size=w/40, text_opacity=0.8, duration_marquee=15.0,
            duration_still=10.0, comment_filter=None, is_reduce_comments=False,
            progress_callback=None)



import json
import re
import os

from bilili.common.subtitle import Subtitle
from bilili.tools import spider


subtitle_api = "https://api.bilibili.com/x/player.so?id=cid:{cid}&aid={avid}&bvid={bvid}"


def get_subtitle(container):
    cid, avid, bvid = container.meta["cid"], container.meta["avid"], container.meta["bvid"]
    # 检查是否有字幕并下载
    subtitle_url = subtitle_api.format(avid=avid, cid=cid, bvid=bvid)
    res = spider.get(subtitle_url)
    subtitles_info = json.loads(
        re.search(r"<subtitle>(.+)</subtitle>", res.text).group(1))
    for sub_info in subtitles_info["subtitles"]:
        sub_path = os.path.splitext(
            container.path)[0] + sub_info["lan_doc"] + ".srt"
        subtitle = Subtitle(sub_path)
        for sub_line in spider.get("https:"+sub_info["subtitle_url"]).json()["body"]:
            subtitle.write_line(
                sub_line["content"], sub_line["from"], sub_line["to"])



import re

from bilili.common.aria2 import Aria2
from bilili.common.ffmpeg import FFmpeg
from bilili.common.crawler import BililiCrawler

aria2 = Aria2(show_progress=True)
ffmpeg = FFmpeg()
spider = BililiCrawler()

regex_acg_video_av = re.compile(r"https?://www.bilibili.com/video/av(?P<avid>\d+)")
regex_acg_video_av_short = re.compile(r"https?://b23.tv/av(?P<avid>\d+)")
regex_acg_video_bv = re.compile(r"https?://www.bilibili.com/video/(bv|BV)(?P<bvid>\w+)")
regex_acg_video_bv_short = re.compile(r"https?://b23.tv/(bv|BV)(?P<bvid>\w+)")
regex_bangumi = re.compile(r"https?://www.bilibili.com/bangumi/media/md(\d+)")



import queue
import threading


class ThreadPool():
    """ 线程池类
    快速创建多个相同任务的线程池
    """
    def __init__(self, num):
        self.num = num
        self._taskQ = queue.Queue()
        self.threads = []

    def add_task(self, task):
        """ 添加任务　"""
        self._taskQ.put(task)

    def _run_task(self):
        """ 启动任务线程　"""
        while True:
            if not self._taskQ.empty():
                task = self._taskQ.get(block = True, timeout = 1)
                task()
                self._taskQ.task_done()
            else:
                break

    def run(self):
        """ 启动线程池　"""
        for _ in range(self.num):
            th = threading.Thread(target=self._run_task)
            th.setDaemon(True)
            self.threads.append(th)
            th.start()

    def join(self):
        """ 等待所有任务结束　"""
        self._taskQ.join()



import os
import re

import requests


class Crawler(requests.Session):

    header = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/71.0.3578.98 Safari/537.36',
    }

    def __init__(self):
        super().__init__()
        self.headers.update(Crawler.header)

    def set_cookies(self, cookies):
        """传入一个字典，用于设置 cookies"""

        requests.utils.add_dict_to_cookiejar(self.cookies, cookies)

    def download_bin(self, url, file_path, stream=True, chunk_size=1024, **kw):
        """下载二进制文件"""

        res = self.get(url, stream=stream, **kw)
        tmp_path = file_path + ".t"
        try:
            with open(tmp_path, "wb") as f:
                if stream:
                    for chunk in res.iter_content(chunk_size=chunk_size):
                        if not chunk:
                            break
                        f.write(chunk)
                else:
                    f.write(res.content)
        except:
            os.remove(tmp_path)
            print("[warn] {} failed to download".format(file_path))
        if os.path.exists(file_path):
            with open(tmp_path, "rb") as fr:
                with open(file_path, "wb") as fw:
                    fw.write(fr.read())
            os.remove(tmp_path)
        else:
            os.rename(tmp_path, file_path)

    def download_text(self, url, file_path, **kw):
        """下载文本，以 UTF-8 编码保存文件"""

        res = self.get(url, **kw)
        res.encoding = res.apparent_encoding
        with open(file_path, 'w', encoding='utf_8') as f:
            f.write(res.text)


class BililiCrawler(Crawler):

    header = {
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/64.0.3282.167 Safari/537.36',
        'Referer': 'https://www.bilibili.com',
    }

    def __init__(self):
        super().__init__()
        self.headers.update(BililiCrawler.header)



from bilili.common.base import Text


class Subtitle(Text):
    """ 播放列表类 """

    def __init__(self, path):
        super().__init__(path)
        self._count = 0

    @staticmethod
    def time_format(seconds):
        ms = int(1000 * (seconds - int(seconds)))
        seconds = int(seconds)
        minutes, sec = seconds // 60, seconds % 60
        hour, min = minutes // 60, minutes % 60
        return "{:02}:{:02}:{:02},{}".format(hour, min, sec, ms)


    def write_line(self, content, from_time, to_time):
        self._count += 1
        self.write_string(str(self._count))
        self.write_string(
            "{} --> {}".format(self.time_format(from_time), self.time_format(to_time)))
        self.write_string(content + "\n")



import os
import subprocess


class Aria2():
    def __init__(self, aria2_path='aria2c', show_progress=False):
        self.show_progress = show_progress
        assert subprocess.run([aria2_path], stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE).returncode == 1, "请配置正确的 aria2 路径"
        self.path = os.path.normpath(aria2_path)
        tmp_dir = os.path.join(os.path.dirname(aria2_path), "tmp")
        if not os.path.exists(tmp_dir):
            os.makedirs(tmp_dir)
        self.tmp_dir = os.path.normpath(tmp_dir)

    def exec(self, params):
        """ 调用 aria2 """

        if self.show_progress:
            out_pipe = None
        else:
            out_pipe = subprocess.PIPE

        cmd = [self.path]
        cmd.extend(params)

        return subprocess.run(cmd, stdout=out_pipe, stderr=out_pipe)

    def download_video_list(self, video_list, dirname):
        input_file = os.path.join(self.tmp_dir, "aria2.in")

        with open(input_file, "w", encoding="utf-8") as f:
            for video in video_list:
                f.write(
f'''{video["url"]}
    dir={dirname}
    out={video["filename"]}
    referer=https://www.bilibili.com
    user-agent=Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/64.0.3282.167 Safari/537.36
''')

        command = [
            "aria2c",
            "-i", input_file,
        ]
        self.exec(command)
        os.remove(input_file)

    def download(self, url, filepath):
        dirname, filename = os.path.split(filepath)
        command = [
            "aria2c", url,
            "-d", dirname,
            "-o", filename,
            "--referer", "https://www.bilibili.com",
            "--user-agent", "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/64.0.3282.167 Safari/537.36"
        ]
        self.exec(command)





import os
import re
import time
import math
import subprocess


class BililiContainer():
    """ bilibili 单个视频类
    包括多个 B 站视频已经分好的段
    """

    def __init__(self, id, name, path, meta, format="flv"):

        self.id = id
        self.name = name
        self.path = path
        self.meta = meta
        self.format = format

        self.medias = []
        self.qn = None

    def merge(self, ffmpeg):
        if self.format == 'mp4':
            with open(self.medias[0].path, 'rb') as fr:
                with open(self.path, 'wb') as fw:
                    fw.write(fr.read())
        elif self.format == 'flv':
            video_path_list = [media.path for media in self.medias]
            ffmpeg.join_videos(video_path_list, self.path)
        elif self.format == 'm4s':
            ffmpeg.join_video_audio(self.medias[0].path, self.medias[1].path, self.path)
        else:
            print("Unknown format {}".format(self.format))
        # 清除合并完成的视频片段
        for media in self.medias:
            os.remove(media.path)

    def append_media(self, id, url, qn, type):
        self.medias.append(BililiMedia(
            id = id,
            url = url,
            qn = qn,
            container = self,
            type = type,
        ))

    def download_check(self, overwrite):
        if os.path.exists(self.path) and overwrite:
            os.remove(self.path)
            return True
        elif os.path.exists(self.path) and not overwrite:
            return False
        else:
            return True


class BililiMedia():

    def __init__(self, id, url, qn, container, type="segment"):

        self.id = id
        self.qn = qn
        self.url = url
        self.container = container
        self.path = os.path.splitext(self.container.path)[0]
        if self.container.format == "flv":
            self.path += "_{:02d}.flv".format(id)
        elif self.container.format == "m4s":
            self.path += "_{}.m4s".format(type)
        elif self.container.format == "mp4":
            self.path += "_dl.mp4"
        self.tmp_path = self.path + ".dl"
        self.name = os.path.split(self.path)[-1]
        self.tmp_name = os.path.split(self.tmp_path)[-1]

        if self.container.qn is None:
            self.container.qn = qn

    def rename(self):
        if os.path.exists(self.path):
            os.remove(self.path)
        os.rename(self.tmp_path, self.path)

    def download_check(self, overwrite):
        if os.path.exists(self.path) and overwrite:
            os.remove(self.path)
            return True
        elif os.path.exists(self.path) and not overwrite:
            return False
        else:
            return True



import os

from bilili.common.base import Text


class Playlist(Text):
    """ 播放列表类 """

    def __init__(self, path, path_type):
        super().__init__(path)
        self.path_type = path_type

    def switch_path(self, path):
        path = os.path.normpath(path)
        if self.path_type == 'AP':
            path = os.path.abspath(path)
        elif self.path_type == 'RP':
            path = os.path.relpath(path, start=os.path.dirname(self.path))
        return path

    def write_path(self, path):
        path = self.switch_path(path)
        self.write_string(path)


class M3u(Playlist):
    """ m3u 播放列表 """

    def __init__(self, path, path_type='RP'):
        super().__init__(path, path_type)


class Dpl(Playlist):
    """ potplayer 播放列表 """

    def __init__(self, path, path_type='RP'):
        super().__init__(path, path_type)
        self.write_string('DAUMPLAYLIST\n')
        self._count = 0

    def write_path(self, path, name=None):
        self._count += 1
        path = self.switch_path(path)
        self.write_string('{}*file*{}'.format(self._count, path))
        if name is not None:
            self.write_string('{}*title*{}\n'.format(self._count, name))



from setuptools import setup, find_packages

VERSION = '0.0.9'

def get_long_description():
    with open('README.md', 'r', encoding='utf-8') as f:
        desc = f.read()
    return desc

setup(
    name='bilili',
    version=VERSION,
    description=":beers: bilibili video and danmaku downloader | B站视频、弹幕下载器",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
    keywords='python bilibili video download spider danmaku',
    author='SigureMo',
    author_email='sigure.qaq@gmail.com',
    url='https://github.com/SigureMo/bilili',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=True,
    install_requires=[
        'requests',
        'opencv-python'
    ],
    entry_points={
        'console_scripts':[
            'bilili = bilili.bilili_dl:main'
        ]
    },
)



import re
import sys
import argparse
import os
import json

from bilili.utils import parse_episodes, convert_danmaku
from bilili.common.base import repair_filename, touch_dir, remove
from bilili.common.playlist import Dpl, M3u
from bilili.api.subtitle import get_subtitle
from bilili.api.danmaku import get_danmaku
from bilili.tools import (aria2, ffmpeg, spider, regex_acg_video_av, regex_acg_video_av_short,
                            regex_acg_video_bv, regex_acg_video_bv_short, regex_bangumi)


def main():
    """ 解析命令行参数并调用相关模块进行下载 """

    parser = argparse.ArgumentParser(description="bilili B 站视频、弹幕下载器")
    parser.add_argument("url", help="视频主页地址")
    parser.add_argument('-f', '--format', default='m4s',
                        choices=['flv', 'm4s', 'mp4'], help="选择播放源（html5 or flash or mp4）")
    parser.add_argument("-d", "--dir", default=r"", help="下载目录")
    parser.add_argument("-q", "--quality", default='120', choices=['120', '116', '112', '80', '74', '64', '32', '16', '6'],
                        help="视频清晰度 112:1080P+, 80:1080P, 64:720P, 32:480P, 16:360P")
    parser.add_argument("-p", "--episodes", default="all", help="选集")
    parser.add_argument("-w", "--overwrite",
                        action="store_true", help="强制覆盖已下载视频")
    parser.add_argument("-c", "--sess-data", default=None, help="输入 cookies")
    parser.add_argument("--ass", action="store_true",
                        help="自动将 xml 弹幕转换为 ass 弹幕")
    parser.add_argument("--playlist-type", default="dpl",
                        choices=["dpl", "m3u", "no"], help="播放列表类型，支持 dpl 和 m3u，输入 no 不生成播放列表")
    parser.add_argument("--path-type", default="rp",
                        help="播放列表路径类型（rp：相对路径，ap：绝对路径）")

    args = parser.parse_args()
    # 超清 4K 高清 1080P60 高清 1080P+ 高清 1080P  高清 720P60 高清 720P  清晰 480P  流畅 360P 极速 240P
    quality_lists = [120, 116, 112, 80, 74, 64, 32, 16, 6]
    cookies = {
        "SESSDATA": args.sess_data
    }

    config = {
        "url": args.url,
        "dir": args.dir,
        "quality_sequence": quality_lists[quality_lists.index(int(args.quality)):] +
        list(reversed(quality_lists[:quality_lists.index(int(args.quality))])),
        "episodes": args.episodes,
        "playlist_type": args.playlist_type,
        "playlist_path_type": args.path_type.upper(),
        "overwrite": args.overwrite,
        "cookies": cookies,
        "format": args.format.lower(),
    }

    if regex_acg_video_av.match(args.url) or \
       regex_acg_video_av_short.match(args.url) or \
       regex_acg_video_bv.match(args.url) or \
       regex_acg_video_bv_short.match(args.url):
        bili_type = "acg_video"
        from bilili.api.acg_video import get_title, get_context, get_containers, parse_segments
    elif regex_bangumi.match(args.url):
        bili_type = "bangumi"
        from bilili.api.bangumi import get_title, get_context, get_containers, parse_segments
    else:
        print("视频地址有误！")
        sys.exit(1)

    home_url = args.url

    # 获取标题
    spider.set_cookies(config["cookies"])
    title = get_title(home_url)
    print(title)

    # 创建所需目录结构
    base_dir = touch_dir(os.path.join(
        config['dir'], repair_filename(title + " - bilibili")))
    video_dir = touch_dir(os.path.join(base_dir, "Videos"))
    if args.overwrite:
        remove(video_dir)
        touch_dir(video_dir)
    if config['playlist_type'] == 'dpl':
        playlist = Dpl(os.path.join(base_dir, 'Playlist.dpl'),
                       path_type=config["playlist_path_type"])
    elif config["playlist_type"] == "m3u":
        playlist = M3u(os.path.join(base_dir, 'Playlist.m3u'),
                       path_type=config["playlist_path_type"])
    else:
        playlist = None

    # 获取需要的信息
    context = get_context(home_url)
    containers = get_containers(context, video_dir, config['format'], playlist)

    # 解析并过滤不需要的选集
    episodes = parse_episodes(config["episodes"], len(containers))
    containers = list(filter(lambda video: video.id in episodes, containers))

    # 解析片段信息及视频 url
    for i, container in enumerate(containers):
        print("{:02}/{:02} parsing segments info...".format(i,
                                                            len(containers)), end="\r")
        if bili_type == 'acg_video':
            get_subtitle(container)
        get_danmaku(container)
        parse_segments(container, config['quality_sequence'])

    if containers:
        containers_need_download = []
        medias_need_download = []
        for i, container in enumerate(containers):
            if container.download_check(overwrite=args.overwrite):
                containers_need_download.append(container)
                sign = "✓"
                print("{} {} qn: {}".format(sign, container.name, container.qn))
                for media in container.medias:
                    if media.download_check(overwrite=args.overwrite):
                        medias_need_download.append(media)
                        sign = "✓"
                        print("    {} {}".format(sign, media.name))
                    else:
                        sign = "✖"
                        print("    {} {}".format(sign, media.name))
            else:
                sign = "✖"
                print("{} {}".format(sign, container.name))

        aria2.download_video_list(map(lambda media: {
                "url": media.url,
                "filename": media.tmp_name
            }, medias_need_download), video_dir)
        for i, media in enumerate(medias_need_download):
            print("renaming {} {}/{}".format(media.name, i +
                                            1, len(medias_need_download)), end="\r")
            media.rename()
        for i, container in enumerate(containers_need_download):
            print("merging {} {}/{}".format(container.name, i +
                                            1, len(containers_need_download)), end="\r")
            container.merge(ffmpeg)
        print("已全部下载完成！")
    else:
        print("没有需要下载的视频！")

    # 弹幕转换为 ass 格式
    if args.ass:
        convert_danmaku([
            container.path for container in containers
        ])
        print("转换完成")


if __name__ == "__main__":
    main()



import pytest

from bilili.api.bangumi import get_title, get_context, get_containers, parse_segments
from bilili.common.base import touch_dir
from bilili.video import BililiContainer


def test_get_title():
    url = "https://www.bilibili.com/bangumi/media/md28223066/"
    assert get_title(url) == "我的三体之章北海传"

def test_get_context():
    url = "https://www.bilibili.com/bangumi/media/md28223066/"
    context = get_context(url)
    assert context["season_id"] == '28770'

def test_get_containers():
    context = {
        "season_id": '28770',
    }
    video_dir = touch_dir("tmp/ThreeBody/Videos/")
    containers = get_containers(context, video_dir, 'flv', None)
    assert len(containers) == 9

@pytest.mark.parametrize(
    'format', [
        'flv', 'm4s'
    ])
def test_parse_segments(format):
    container = BililiContainer(
        id=1,
        name="Unknown",
        path=touch_dir("tmp/ThreeBody/Videos/video1"),
        meta={
            "aid": 84271171,
            "cid": 144541892,
            "epid": 300998,
            "bvid": '',
        },
        format=format,
    )
    parse_segments(container, [120, 116, 112, 80, 74, 64, 32, 16, 6])
